local GetUnitSpeed = GetUnitSpeed
local IsFlying = IsFlying

local db


-- data for the various unit displays
local UnitData = {
	percent = {
		display = "%",
		width = 64,
		name = "percentage",
	},
	yards = {
		display = " yd/s",
		width = 80,
		name = "yards per second",
	},
	miles = {
		display = " mph",
		width = 80,
		name = "miles per hour",
	},
	kilometers = {
		display = " km/h",
		width = 88,
		name = "kilometers per hour",
	},
	meters = {
		display = " m/s",
		width = 80,
		name = "meters per second",
	},
}


-- create the speed-o-meter frame
local speedOMeter = CreateFrame("Frame", "SpeedyGonzalesFrame", UIParent)
speedOMeter:SetToplevel(true)
speedOMeter:SetFrameStrata("MEDIUM")
speedOMeter:SetPoint("CENTER", 0, -100)
speedOMeter:SetHeight(32)
speedOMeter:EnableMouse(true)
speedOMeter:SetMovable(true)
speedOMeter:SetBackdrop(
	{
		bgFile = ("Interface\\ChatFrame\\ChatFrameBackground"),
		edgeFile = ("Interface\\DialogFrame\\UI-DialogBox-Border"),
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 4, right = 4, top = 4, bottom = 4 }
	}
)
speedOMeter:SetBackdropColor(0, 0, 0, 0.8)
speedOMeter:RegisterEvent("ADDON_LOADED")

-- create font string for the actual speed text
local speedOutput = speedOMeter:CreateFontString("SpeedyGonzalesFrameOutput")
speedOutput:SetPoint("CENTER", speedOMeter)
speedOutput:SetFontObject(GameFontHighlight)


function SpeedyGonzales_OnEvent(self, event, ...)
	if select(1, ...) == "SpeedyGonzales" then
		-- initiate saved variables if they do not exist
		if not (SpeedyGonzalesData and SpeedyGonzalesData.units) then
			SpeedyGonzalesData = {
				units = "percent",
				pitch = false,
				locked = false,
				shown = true,
			}
		end
		db = SpeedyGonzalesData
		SpeedyGonzales_SetVisibility()
		SpeedyGonzales_SetWidth()
		SpeedyGonzales_SetLock()
		SpeedyGonzales_SetPitchDisplay()
	end
end

speedOMeter:SetScript("OnEvent", SpeedyGonzales_OnEvent)


function SpeedyGonzales_OnUpdate()
	local text = SpeedyGonzales_GetSpeed()..UnitData[db.units]["display"]
	if db.pitch then
		if IsFlying() then
			text = format("%.1f \194\176\n", deg(GetUnitPitch("player")))..text
		else
			text = "-\n"..text
		end
	end
	speedOutput:SetText(text)
end

speedOMeter:SetScript("OnUpdate", SpeedyGonzales_OnUpdate)


function SpeedyGonzales_SlashHandler(msg)
	local arg = {}
	for v in gmatch(strlower(msg), "[^ ]+") do
		arg[#arg + 1] = v
	end
	
	if arg[1] == "units" then
		-- set unit display
		if UnitData[arg[2]] then
			db.units = arg[2]
			print("Now displaying speed in "..UnitData[db.units]["name"]..".")
			SpeedyGonzales_SetWidth()
		else
			print("Invalid unit specified. Please choose one of percent, yards, miles, kilometers or meters.")
		end
	elseif arg[1] == "pitch" then
		-- toggle pitch display
		db.pitch = not db.pitch
		SpeedyGonzales_SetPitchDisplay()
		print("Pitch display "..(db.pitch and "on" or "off")..".")
	elseif arg[1] == "lock" then
		-- toggle frame lock
		db.locked = not db.locked
		SpeedyGonzales_SetLock()
		print("SpeedyGonzales frame "..(db.locked and "locked" or "unlocked")..".")
	else
		-- toggle visibility
		db.shown = not db.shown
		SpeedyGonzales_SetVisibility()
	end
end

SlashCmdList["SPEEDYGONZALES"] = SpeedyGonzales_SlashHandler
SLASH_SPEEDYGONZALES1 = "/speedy"


function SpeedyGonzales_SetVisibility()
	if db.shown then
		speedOMeter:Show()
	else
		speedOMeter:Hide()
	end
end


-- set width depending on displayed unit type
function SpeedyGonzales_SetWidth()
	speedOMeter:SetWidth(UnitData[db.units]["width"])
end


function SpeedyGonzales_SetLock()
	local scriptDown, scriptUp
	
	if not db.locked then
		scriptDown = function(self) self:StartMoving() end
		scriptUp = function(self) self:StopMovingOrSizing() end
	end
	
	speedOMeter:SetScript("OnMouseDown", scriptDown)
	speedOMeter:SetScript("OnMouseUp", scriptUp)
end


-- resize frame so the pitch display fits
function SpeedyGonzales_SetPitchDisplay()
	if db.pitch then
		speedOMeter:SetHeight(48)
	else
		speedOMeter:SetHeight(32)
	end
end


function SpeedyGonzales_GetSpeed()
	local speed = GetUnitSpeed("player")
	
	if db.units == "percent" then
		speed = floor(speed / 7 * 100 + 0.1)
	elseif db.units == "yards" then
		speed = floor(speed * 10 + 0.01) / 10
	elseif db.units == "miles" then
		speed = floor(speed / 1.76 * 36 + 0.01) / 10
	elseif db.units == "kilometers" then
		speed = floor(speed * 9.144 * 3.6 + 0.01) / 10
	elseif db.units == "meters" then
		speed = floor(speed * 9.144 + 0.01) / 10
	end
	
	return speed
end